-- * PLSQL:       co2_gridding_pkg.sql Version 1.1 02/23/2016
-- * Author:      John R. Victorine
-- * Description: This Package will build the Brine Data XML data stream
-- *              for gridding and mapping.
-- * Copyright (c) 2016 Kansas Geological Survey
-- * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
-- * All Rights Reserved.
--
SET SCAN OFF;
--
CREATE OR REPLACE PACKAGE co2_gridding_pkg AS
  PROCEDURE getDateXML(sID IN VARCHAR2);
  PROCEDURE add_fluid_data(sKEY IN VARCHAR2);
  PROCEDURE dtd_and_columns;
--
  PROCEDURE getWellXML(sID IN VARCHAR2);
  PROCEDURE getDataKGS(sKID IN VARCHAR2);
  PROCEDURE getData(sKID IN VARCHAR2);
END co2_gridding_pkg;
/
--
CREATE OR REPLACE PACKAGE BODY co2_gridding_pkg AS
--
-- PROCEDURE    getDateXML
-- Description: This procedure will get brine data as xml by formation.  
-- Input:       iDate = the date to filter on
--              sTime = Formation Name
--              field = Field Name
--
  PROCEDURE getDateXML(sID IN VARCHAR2) IS
--
    l_count    NUMBER;
    o_kid      iqstrat.co2_brine_sample.kid%TYPE;        -- Sample KID
    o_wkid     iqstrat.co2_brine_sample.well_kid%TYPE;   -- Well KID
    o_group    iqstrat.co2_brine_sample.group_no%TYPE;   -- Group Number
--
   CURSOR crsr(sID IN VARCHAR2) IS
     SELECT kid, group_no
       FROM co2_brine_sample
      WHERE group_id = sID
     ORDER BY group_no, sampled_date, well_kid;
--
  BEGIN
--
--  ADD DTD and Fluid Data Columns to XML
--
    dtd_and_columns;
--
    SELECT COUNT(kid)
      INTO l_count
      FROM co2_brine_sample
     WHERE group_id = sID
    ORDER BY group_no, sampled_date, well_kid;
--
    htp.print('<fluid_data>');
    htp.print('<data records="'||l_count||'">');
--
--  GET Fluid Data Record by Brine Data Primary Key
--
    OPEN crsr(sID);
    LOOP
      FETCH crsr
       INTO o_kid,   -- Sample KID
            o_group; -- Group Number
      EXIT WHEN crsr%NOTFOUND;
--      
      add_fluid_data(o_kid);     
--
    END LOOP;
    CLOSE crsr;
--    
    htp.print('</data>');
    htp.print('</fluid_data>');
--       
  EXCEPTION
    WHEN OTHERS THEN
      htp.p('An error occurred: '||SQLERRM);
  END getDateXML;
--
--
-- PROCEDURE    add_fluid_data
-- Description: This procedure will build the DTD and the Columns in the 
--              brine data table.
  PROCEDURE add_fluid_data(sKEY IN VARCHAR2) IS
--
  o_count         NUMBER;
--
-- LOCATION Variables
  o_kid           brine.samples.kid%TYPE;
  o_well_kid      co2_brine_well.kid%TYPE;
  o_api           co2_brine_well.api_number%TYPE;
  o_lease         brine.samples.lease_name%TYPE;
  o_well          brine.samples.well_name%TYPE;
  o_state_code    brine.samples.state_code%TYPE;                     
  o_state         global.states.name%TYPE;
  o_county_code   brine.samples.county_code%TYPE; 
  o_county        global.counties.name%TYPE;
  o_field_code    brine.samples.field_code%TYPE;
  o_field         nomenclature.fields.field_name%TYPE;
  o_latitude      brine.samples.latitude%TYPE;
  o_longitude     brine.samples.longitude%TYPE;
  o_utm_x         co2_brine_wells.nad27_utm_x%TYPE; 
  o_utm_y         co2_brine_wells.nad27_utm_y%TYPE; 
  o_utm_zone      co2_brine_wells.nad27_utm_zone%TYPE; 
  o_location      VARCHAR2(255); 
-- Elevation Data  
  o_kb            brine.samples.elevation_kb%TYPE;
  o_gl            brine.samples.elevation_gl%TYPE;
  o_df            brine.samples.elevation_df%TYPE;
  o_tgt           brine.samples.elevation_tgt%TYPE; 
-- DEPTH Variables
  o_depth_top     brine.samples.depth_top%TYPE;                      
  o_depth_base    brine.samples.depth_base%TYPE;                     
--
  o_mnem          VARCHAR2(60);
  o_value         iqstrat.co2_brine_data.value%TYPE;
  o_units         iqstrat.co2_brine_data.units%TYPE;
--
  o_temp          VARCHAR(4096);
--
  o_group_id      co2_brine_sample.group_id%TYPE;
  o_group_no      co2_brine_sample.group_no%TYPE;
--
  o_lab_kid       iqstrat.co2_brine_sample.kid%TYPE;
  o_date          VARCHAR2(40);
--  
  o_name          iqstrat.CO2_brine_laboratory.name%TYPE;
  o_address       iqstrat.CO2_brine_laboratory.address%TYPE;
  o_city          iqstrat.CO2_brine_laboratory.city%TYPE;
  o_state_lab     VARCHAR2(60);
  o_zipcode       iqstrat.CO2_brine_laboratory.zipcode%TYPE; 
--
    CURSOR crsrh(sKID IN VARCHAR2) IS
      SELECT kid,
             well_kid,
--             (SELECT name FROM co2_brine_wells WHERE kid = CBS.well_KID),
             NVL(lab_kid, '-99'),
             TO_CHAR(sampled_date, 'MM/DD/YYYY HH24:MI'),
             NVL(group_id, '-99'),
             NVL(group_no, -99)
        FROM co2_brine_sample CBS
       WHERE kid = sKID;
--
    CURSOR crsr(sKID IN VARCHAR2) IS
      SELECT ( SELECT mnemonic FROM co2_brine_data_type WHERE id = CBD.data_type_kid ), 
             value,
             units
        FROM co2_brine_data CBD
       WHERE sample_kid = sKID
      ORDER BY data_type_kid;
--    
  BEGIN
--
    OPEN crsrh(sKEY);
    LOOP
--
      FETCH crsrh
       INTO o_kid,      -- Sample KID
--            o_lease,   -- Well Lease Name
            o_well_kid, -- Well header kid
            o_lab_kid,  -- Laboratory KID
            o_date,     -- Sample Date
            o_group_id, -- Group ID
            o_group_no; -- Group Number
      EXIT WHEN crsrh%NOTFOUND;
--
      SELECT COUNT(kid)
        INTO o_count
        FROM co2_brine_well
       WHERE kid = o_well_kid;  
--
      IF (o_count > 0) THEN
        SELECT REPLACE(lease_name||' '||well_name,', ',' '),
               api_number,
               NVL(elevation_kb, -99),
               NVL(elevation_gl, -99),
               NVL(elevation_df, -99),
               NVL(rotary_total_depth, -99),
               nad27_latitude,
               nad27_longitude,
               nad27_utm_x,
	       nad27_utm_y,
	       nad27_utm_zone
          INTO o_lease,
               o_api,
               o_kb,
               o_gl,
               o_df,
               o_tgt,
               o_latitude,
               o_longitude,
               o_utm_x, 
               o_utm_y, 
               o_utm_zone
          FROM co2_brine_well
         WHERE kid = o_well_kid;  
      ELSE
        SELECT REPLACE(name,', ',' '),
               api_number,
               NVL(elevation_kb, -99),
               NVL(elevation_gl, -99),
               NVL(elevation_df, -99),
               NVL(rotary_total_depth, -99),
               nad27_latitude,
               nad27_longitude,
               nad27_utm_x,
	       nad27_utm_y,
	       nad27_utm_zone
          INTO o_lease,
               o_api,
               o_kb,
               o_gl,
               o_df,
               o_tgt,
               o_latitude,
               o_longitude,
               o_utm_x, 
               o_utm_y, 
               o_utm_zone
          FROM co2_brine_wells 
         WHERE kid = o_well_kid;
      END IF;

      IF (o_lab_kid != '-99') THEN
        SELECT NVL(REPLACE(name,'&','&amp;'), '-99'), 
               NVL(REPLACE(address,'&','&amp;'), '-99'), 
               NVL(city, '-99'),
               NVL((SELECT name 
                      FROM global.states 
                     WHERE code = CBL.state_code), '-99'),
               NVL(zipcode, '-99')
          INTO o_name,
               o_address,
               o_city,
               o_state_lab,
               o_zipcode
          FROM CO2_brine_laboratory CBL
         WHERE kid = o_lab_kid;
      END IF;
--
      o_temp := '<row KEY="'||o_kid||'" ';
      o_temp := o_temp||' '||'API="'||o_api||'" NAME="'||o_lease||'" ';
      o_temp := o_temp||' '||'WELL_KID="'||o_well_kid||'" ';
--
      IF (o_kb != -99) THEN
        o_temp := o_temp||'KB="'||o_kb||'" ';
      END IF;
--
      IF (o_gl != -99) THEN
        o_temp := o_temp||'GL="'||o_gl||'" ';
      END IF;
--
      IF (o_df != -99) THEN
        o_temp := o_temp||'DF="'||o_df||'" ';
      END IF;
--
      IF (o_tgt != -99) THEN
        o_temp := o_temp||'TGT="'||o_tgt||'" ';
      END IF;
--
      o_temp := o_temp||' '||'LAT="'||o_latitude||'" LONG="'||o_longitude||'" ';
      o_temp := o_temp||' '||'UTM_X="'||o_utm_x||'" UTM_Y="'||o_utm_y||'" UTM_Z="'||o_utm_zone||'" ';
--
      IF (o_name != '-99') THEN
        o_temp := o_temp||'LAB="'||o_name||'" ';
      END IF;       
--
      IF (o_address != '-99') THEN
        o_temp := o_temp||'ADDRESS="'||o_address||'" ';
      END IF;       
--
      IF (o_city != '-99') THEN
        o_temp := o_temp||'CITY="'||o_city||'" ';
      END IF;       
--
      IF (o_state_lab != '-99') THEN
        o_temp := o_temp||'STATE_L="'||o_state_lab||'" ';
      END IF;       
--
      IF (o_zipcode != '-99') THEN
        o_temp := o_temp||'ZIPCODE="'||o_zipcode||'" ';
      END IF;       
--
      o_temp := o_temp||'DATE="'||o_date||'" ';
      IF (o_group_id != '-99') THEN
        o_temp := o_temp||'ID="'||o_group_id||'" ';
      END IF;       
      IF (o_group_no != -99) THEN
        o_temp := o_temp||'NO="'||o_group_no||'" ';
      END IF;
--
    END LOOP;
    CLOSE crsrh;
--
    OPEN crsr(sKEY);
    LOOP
--    
      FETCH crsr
       INTO o_mnem,  -- Mnemonic
            o_value, -- Value of the Data
            o_units; -- Unit of the Data
      EXIT WHEN crsr%NOTFOUND;
--
      o_temp := o_temp||' '||o_mnem||'="'||o_value||' '||o_units||'" ';
--      htp.print(o_mnem||'="'||o_value||' '||o_units||'" ');
--
    END LOOP;
    CLOSE crsr;
--
--    htp.print('/>');
    htp.print(o_temp||'/>');
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.p('An error occurred: '||SQLERRM||'. Please try again later. '||sKEY);
  END add_fluid_data;
--
--
-- PROCEDURE    dtd_and_columns
-- Description: This procedure will build the DTD and the Columns in the 
--              fluid data table.
-- INPUT:       sKEY = the Brine Data Sample Primary KEY
--
  PROCEDURE dtd_and_columns(sKEY IN VARCHAR2) IS
--
  o_kid  iqstrat.co2_brine_data_type.id%TYPE;
  o_mnem iqstrat.co2_brine_data_type.mnemonic%TYPE;
--
    CURSOR crsr(sKID IN VARCHAR2) IS
      SELECT cbdt.id,
             cbdt.mnemonic
        FROM co2_brine_data_type cbdt,
             co2_brine_data CBD
       WHERE CBD.sample_kid = sKID
         AND CBD.data_type_kid  = cbdt.id
       ORDER BY cbdt.id;
--
  BEGIN
--
  htp.print('<?xml version="1.0"?>');
  htp.print('<!DOCTYPE fluid_data [');
  htp.print('<!ELEMENT fluid_data (data*)+>');
  htp.print('<!ELEMENT data (row*)>');
  htp.print('<!ATTLIST data records CDATA #REQUIRED>');
  htp.print('<!ELEMENT row EMPTY>');
-- Primary Keys
  htp.print('<!ATTLIST row  KEY     CDATA #REQUIRED'); -- Fluid Primary Key
  htp.print('               STATE   CDATA #IMPLIED');  -- State 
  htp.print('               COUNTY  CDATA #IMPLIED');  -- County 
  htp.print('               FIELD   CDATA #IMPLIED');  -- Field  
  htp.print('               LOC     CDATA #IMPLIED');  -- Location - TRS  
  htp.print('               API     CDATA #IMPLIED');  -- Well API-Number
  htp.print('               NAME    CDATA #IMPLIED');  -- Well Name
  htp.print('               WELL_KID CDATA #IMPLIED'); -- Well KID
-- Elevation
  htp.print('               KB      CDATA #IMPLIED');  -- Kelly Bushing
  htp.print('               GL      CDATA #IMPLIED');  -- Ground Level
  htp.print('               DF      CDATA #IMPLIED');  -- Derrick Floor
  htp.print('               TGT     CDATA #IMPLIED');  -- TGT  
-- Location Information
  htp.print('               LAT     CDATA #IMPLIED');  -- Latitude
  htp.print('               LONG    CDATA #IMPLIED');  -- Longitude
  htp.print('               UTM_X   CDATA #IMPLIED');  -- UTM X
  htp.print('               UTM_Y   CDATA #IMPLIED');  -- UTM Y
  htp.print('               UTM_Z   CDATA #IMPLIED');  -- UTM Zone
-- Formation Information
  htp.print('               SRC     CDATA #IMPLIED');  -- Sample Source  
--  
  htp.print('               LAB     CDATA #IMPLIED');  -- Lab Name
  htp.print('               ADDRESS CDATA #IMPLIED');  -- Lab Address
  htp.print('               CITY    CDATA #IMPLIED');  -- City
  htp.print('               STATE_L CDATA #IMPLIED');  -- State
  htp.print('               ZIPCODE CDATA #IMPLIED');  -- Zipcode
--  
  htp.print('               DATE    CDATA #IMPLIED');  -- Sample Date  
-- Sample Information
--  htp.print('               FORM    CDATA #IMPLIED');  -- Formation
--  htp.print('               AGE     CDATA #IMPLIED');  -- Formation Age
-- Depth Information
--  htp.print('               TOP     CDATA #IMPLIED');  -- Depth Top
--  htp.print('               BASE    CDATA #IMPLIED');  -- Depth Base
--
    OPEN crsr(sKEY);
    LOOP
      FETCH crsr
       INTO o_kid,   -- Data Type ID
            o_mnem;  -- Data Type Mnemonic
      EXIT WHEN crsr%NOTFOUND;
--
      htp.print('               '||o_mnem||'    CDATA #IMPLIED');
--
    END LOOP;
    CLOSE crsr;
--
  htp.print('>]>');
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.p('An error occurred: '||SQLERRM||'. Please try again later.');
  END dtd_and_columns;
--
--
-- PROCEDURE    dtd_and_columns
-- Description: This procedure will build the DTD and the Columns in the 
--              fluid data table.
  PROCEDURE dtd_and_columns IS
--
  o_kid  iqstrat.co2_brine_data_type.id%TYPE;
  o_mnem iqstrat.co2_brine_data_type.mnemonic%TYPE;
--
    CURSOR crsr IS
      SELECT UNIQUE(cbdt.id)
        FROM co2_brine_data_type cbdt,
             co2_brine_data CBD
       WHERE CBD.data_type_kid  = cbdt.id
       ORDER BY cbdt.id;
--
  BEGIN
--
  htp.print('<?xml version="1.0"?>');
  htp.print('<!DOCTYPE fluid_data [');
  htp.print('<!ELEMENT fluid_data (data*)+>');
  htp.print('<!ELEMENT data (row*)>');
  htp.print('<!ATTLIST data records CDATA #REQUIRED>');
  htp.print('<!ELEMENT row EMPTY>');
-- Primary Keys
  htp.print('<!ATTLIST row  KEY     CDATA #REQUIRED'); -- Fluid Primary Key
  htp.print('               STATE   CDATA #IMPLIED');  -- State 
  htp.print('               COUNTY  CDATA #IMPLIED');  -- County 
  htp.print('               FIELD   CDATA #IMPLIED');  -- Field  
  htp.print('               LOC     CDATA #IMPLIED');  -- Location - TRS  
  htp.print('               API     CDATA #IMPLIED');  -- Well API-Number
  htp.print('               NAME    CDATA #IMPLIED');  -- Well Name
  htp.print('               WELL_KID CDATA #IMPLIED'); -- Well KID
-- Elevation
  htp.print('               KB      CDATA #IMPLIED');  -- Kelly Bushing
  htp.print('               GL      CDATA #IMPLIED');  -- Ground Level
  htp.print('               DF      CDATA #IMPLIED');  -- Derrick Floor
  htp.print('               TGT     CDATA #IMPLIED');  -- TGT  
-- Location Information
  htp.print('               LAT     CDATA #IMPLIED');  -- Latitude
  htp.print('               LONG    CDATA #IMPLIED');  -- Longitude
  htp.print('               UTM_X   CDATA #IMPLIED');  -- UTM X
  htp.print('               UTM_Y   CDATA #IMPLIED');  -- UTM Y
  htp.print('               UTM_Z   CDATA #IMPLIED');  -- UTM Zone
-- Formation Information
  htp.print('               SRC     CDATA #IMPLIED');  -- Sample Source  
--  
  htp.print('               LAB     CDATA #IMPLIED');  -- Lab Name
  htp.print('               ADDRESS CDATA #IMPLIED');  -- Lab Address
  htp.print('               CITY    CDATA #IMPLIED');  -- City
  htp.print('               STATE_L CDATA #IMPLIED');  -- State
  htp.print('               ZIPCODE CDATA #IMPLIED');  -- Zipcode
--  
  htp.print('               DATE    CDATA #IMPLIED');  -- Sample Date  
  htp.print('               ID      CDATA #IMPLIED');  -- Sample Date  
  htp.print('               NO      CDATA #IMPLIED');  -- Sample Date  
-- Sample Information
--  htp.print('               FORM    CDATA #IMPLIED');  -- Formation
--  htp.print('               AGE     CDATA #IMPLIED');  -- Formation Age
-- Depth Information
--  htp.print('               TOP     CDATA #IMPLIED');  -- Depth Top
--  htp.print('               BASE    CDATA #IMPLIED');  -- Depth Base
--
    OPEN crsr;
    LOOP
      FETCH crsr
       INTO o_kid;  -- Data Type ID
      EXIT WHEN crsr%NOTFOUND;
--
      SELECT mnemonic
        INTO o_mnem
        FROM co2_brine_data_type
       WHERE id = o_kid;
--       
      htp.print('               '||o_mnem||'    CDATA #IMPLIED');
--
    END LOOP;
    CLOSE crsr;
--
  htp.print('>]>');
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.p('An error occurred: '||SQLERRM||'. Please try again later.');
  END dtd_and_columns;
--
--
-- PROCEDURE    getWellXML()
-- Description: This procedure will retrieve well_headers information from the
--              well_headers table and construct a XML.
--
  PROCEDURE getWellXML(sID IN VARCHAR2) IS
--
    l_count  NUMBER;
    l_count1 NUMBER;
    l_count2 NUMBER;
    o_kid    user_headers.headers_kid%TYPE; -- headers table primary key
--
    CURSOR crsr(sID IN VARCHAR2) IS
      SELECT UNIQUE(well_kid)
        FROM co2_brine_sample cbs,
             co2_brine_well qwh
       WHERE cbs.well_kid = qwh.kid
         AND cbs.group_id = sID;
--
    CURSOR crsrw(sID IN VARCHAR2) IS
      SELECT UNIQUE(cbw.kid)
        FROM co2_brine_sample cbs,
             co2_brine_wells cbw
       WHERE cbs.well_kid = cbw.kid
         AND cbw.kid NOT IN (1043234355, 1043234370, 1044998939, 1001328003, 1001328011, 1001327997)
         AND cbs.group_id = sID;
--
  BEGIN
--
    SELECT COUNT(UNIQUE(well_kid))
      INTO l_count1
      FROM co2_brine_sample cbs,
           co2_brine_well qwh
     WHERE cbs.well_kid = qwh.kid
       AND cbs.group_id = sID;
--
    SELECT COUNT(UNIQUE(cbw.kid))
      INTO l_count2
      FROM co2_brine_sample cbs,
           co2_brine_wells cbw
     WHERE cbs.well_kid = cbw.kid
       AND cbw.kid NOT IN (1043234355, 1043234370, 1044998939, 1001328003, 1001328011, 1001327997)
       AND cbs.group_id = sID;
--
    l_count := l_count1 + l_count2;
--
--  ADD Headers DTD 
--
    kgs_well_headers_pkg.dtd_and_columns;
--
    IF (l_count > 0) THEN
      htp.print('<headers records="'||l_count||'">');     
--
      OPEN crsr(sID);
      LOOP
        FETCH crsr
         INTO o_kid;    -- headers table primary key
        EXIT WHEN crsr%NOTFOUND;
--
        getDataKGS(o_kid);
--
      END LOOP;
      CLOSE crsr;
--
      OPEN crsrw(sID);
      LOOP
        FETCH crsrw
         INTO o_kid;    -- headers table primary key
        EXIT WHEN crsrw%NOTFOUND;
--
        getData(o_kid);
--
      END LOOP;
      CLOSE crsrw;
--
      htp.print('</headers>');
    ELSE
      htp.print('<headers records="0">');
      htp.print(' <data>');
      htp.print('    <info error="NO DATA AVAILABLE"/>');
      htp.print(' </data>');
      htp.print('</headers>');
    END IF;
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.print(' <data>');
      htp.print('    <info error="'||SQLERRM||'"/>');
      htp.print(' </data>');
      htp.print('</headers>');
--
  END getWellXML;
--
--
-- PROCEDURE    getDataKGS()
-- Description: This procedure will retrieve headers information from the
--              headers table and construct a XML.
-- Input:       sKID = Headers Primary Key of the header information data 
--              sKEY    = User Created Primary Key
--              sUser   = User Primary Key
--              iAccess = User Access to data
--
  PROCEDURE getDataKGS(sKID IN VARCHAR2) IS
--
    l_count      NUMBER;
--    
    o_kid        co2_brine_well.kid%TYPE;        -- Primary Key for table
    o_type       headers.type%TYPE;                      -- Type of header
    o_api        co2_brine_well.api_number%TYPE; -- Well API-Number or UWI 
    o_name       VARCHAR(200);                           -- Well Name or Outcrop Name
    o_status     co2_brine_well.status%TYPE;     -- well status or Outcrop Type  
-- Other well Data Columns
--    o_field      co2_brine_well.field_name%TYPE;    -- Field Name       (well)
--    o_field_kid  co2_brine_well.field_kid%TYPE;     -- Field KID        (well)
--    o_operator   co2_brine_well.operator_name%TYPE; -- Operator Name    (well)
--    o_oper_kid   co2_brine_well.operator_kid%TYPE;  -- Operator KID     (well)
-- Location Columns
--    o_state_cd   co2_brine_well.state_code%TYPE;   -- State Code
--    o_state      global.states.name%TYPE;                  -- State Name
--    o_county_cd  co2_brine_well.county_code%TYPE;  -- County Code
--    o_county     global.states_counties.county_name%TYPE;  -- County Name
    o_location   headers.location%TYPE;                    -- Location 
    o_township   co2_brine_well.township%TYPE;           -- Township
    o_town_dir   co2_brine_well.township_direction%TYPE; -- Township direction
    o_range      co2_brine_well.range%TYPE;              -- Range
    o_range_dir  co2_brine_well.range_direction%TYPE;    -- Range direction
    o_section    co2_brine_well.section%TYPE;            -- Section
-- XY Position of Columns
    o_latitude   co2_brine_well.nad27_latitude%TYPE;     -- Latitude in decimal degrees
    o_longitude  co2_brine_well.nad27_longitude%TYPE;    -- Longitude in decimal degrees
    o_utm_x      co2_brine_well.nad27_utm_x%TYPE;        -- UTM X-Coordinate
    o_utm_y      co2_brine_well.nad27_utm_y%TYPE;        -- UTM Y-Coordinate
    o_utm_zone   co2_brine_well.nad27_utm_zone%TYPE;     -- UTM Zone
-- Z Position - Depth, Elevation Columns
    o_depth      co2_brine_well.rotary_total_depth%TYPE;  -- depth of well or height of outcrop
    o_kb         co2_brine_well.elevation_kb%TYPE;    -- elevation kelly bushing (well)
    o_gl         co2_brine_well.elevation_gl%TYPE;    -- elevation ground level
    o_df         co2_brine_well.elevation_df%TYPE;    -- elevation derrick floor (well)
-- Miscellaneous Columns
--    o_by         co2_brine_well.update_initials%TYPE; -- update by
--    o_date       VARCHAR(20);                                 -- update date
--    o_comments   co2_brine_well.comments%TYPE;        -- general comments
--
    l_cnt        NUMBER(6);  -- Number of LAS Files
    t_cnt        NUMBER(6);  -- Number of tops
    c_cnt        NUMBER(6);  -- Number of Core Data
    i_cnt        NUMBER(6);  -- Number of Image Data
--
    CURSOR crsr (sKID VARCHAR2) IS
      SELECT kid,                    
             'WELL',       
             NVL(api_number, '-99'), 
             NVL(REPLACE((lease_name||' '||well_name),'&','&amp;'), '-99'),
             NVL(REPLACE(status,'&','&amp;'), '-99'),  
--             NVL(REPLACE(field_name,'&','&amp;'), '-99'), 
--             NVL(field_kid, -99), 
--             NVL(REPLACE(operator_name,'&','&amp;'), '-99'),
--             NVL(operator_kid, -99),
--             NVL(state_code, -9),
--             NVL((SELECT name 
--                    FROM global.states 
--                   WHERE code = A.state_code), '-99'), 
--             NVL(county_code, -99),  
--             NVL((SELECT county_name 
--                    FROM global.states_counties 
--                   WHERE state_code = A.state_code
--                     AND county_code = A.county_code), '-99'), 
             NVL(township, -99),  
             NVL(township_direction, 'X'), 
             NVL(range, -99),  
             NVL(range_direction, 'X'), 
             NVL(section, -9),
             NVL(nad27_latitude, -99), 
             NVL(nad27_longitude, -99), 
             NVL(nad27_utm_x, -99), 
             NVL(nad27_utm_y, -99), 
             NVL(nad27_utm_zone, -99), 
             NVL(rotary_total_depth, -99),    
             NVL(elevation_kb, -99),
             NVL(elevation_gl, -99), 
             NVL(elevation_df, -99) 
--             NVL(update_initials, '-99'), 
--             TO_CHAR(update_date, 'MM/DD/YYYY'),
--             NVL(REPLACE(comments,'&','&amp;'), '-99')         
        FROM co2_brine_well A
       WHERE kid = sKID;
--
  BEGIN
--
    o_kid       := '-99';
    o_type      := '-99';  -- Type of header - WELL or OUTCROP
    o_api       := '-99';  -- Well API-Number or UWI 
    o_name      := '-99';  -- Well Name or Outcrop Name
    o_status    := '-99';  -- well status or Outcrop Type  
-- Other well Data Columns
--    o_field     := '-99';  -- Field Name       (well)
--    o_field_kid := -99;    -- Field KID        (well)
--    o_operator  := '-99';  -- Operator Name    (well)
--    o_oper_kid  := -99;    -- Operator KID     (well)
-- Location Columns
--    o_state_cd  := -9;     -- State Code
--    o_state     := '-99';  -- State Name
--    o_county_cd := -99;    -- County Code
--    o_county    := '-99';  -- County Name
    o_location  := '-99';  -- Location 
    o_township  := -99;    -- Township
    o_town_dir  := 'X';    -- Township direction
    o_range     := -99;    -- Range
    o_range_dir := 'X';    -- Range direction
    o_section   := -9;     -- Section
-- XY Position of Columns
    o_latitude  := -99;    -- Latitude in decimal degrees
    o_longitude := -99;    -- Longitude in decimal degrees
    o_utm_x     := -99;    -- UTM X-Coordinate
    o_utm_y     := -99;    -- UTM Y-Coordinate
    o_utm_zone  := -99;    -- UTM Zone
-- Z Position - Depth, Elevation Columns
    o_depth     := -99;    -- depth of well or height of outcrop
    o_kb        := -99;    -- elevation kelly bushing (well)
    o_gl        := -99;    -- elevation ground level
    o_df        := -99;    -- elevation derrick floor (well)
-- Miscellaneous Columns
--    o_by        := '-99';  -- update by
--    o_date      := '-99';  -- update date
--    o_comments  := '-99';  -- general comments
--  
    OPEN crsr(sKID);
    LOOP
      FETCH crsr
       INTO o_kid,       -- Primary Key for table
            o_type,      -- Type of header - WELL or OUTCROP
            o_api,       -- Well API-Number or UWI 
            o_name,      -- Well Name or Outcrop Name
            o_status,    -- well status or Outcrop Type  
            -- Other well Data Columns
--            o_field,     -- Field Name       (well)
--            o_field_kid, -- Field KID        (well)
--            o_operator,  -- Operator Name    (well)
--            o_oper_kid,  -- Operator KID     (well)
            -- Location Columns
--            o_state_cd,  -- State Code
--            o_state,     -- State Name
--            o_county_cd, -- County Code
--            o_county,    -- County Name
            o_township,  -- Township
            o_town_dir,  -- Township direction
            o_range,     -- Range
            o_range_dir, -- Range direction
            o_section,   -- Section
            -- XY Position of Columns
            o_latitude,  -- Latitude in decimal degrees
            o_longitude, -- Longitude in decimal degrees
            o_utm_x,     -- UTM X-Coordinate
            o_utm_y,     -- UTM Y-Coordinate
            o_utm_zone,  -- UTM Zone
            -- Z Position - Depth, Elevation Columns
            o_depth,     -- depth of well or height of outcrop
            o_kb,        -- elevation kelly bushing (well)
            o_gl,        -- elevation ground level
            o_df;        -- elevation derrick floor (well)
            -- Miscellaneous Columns
--            o_by,        -- update by
--            o_date,      -- update date
--            o_comments;  -- general comments
      EXIT WHEN crsr%NOTFOUND;
--
--      Identification Information
--
        htp.print('  <data>');
        htp.print('    <info kid="'||o_kid||'" '||
                            'well_kid="'||o_kid||'" ');
--        
        IF (o_type != '-99') THEN
          htp.print('type="'||o_type||'" ');
        END IF;
--        
        IF (o_api != '-99') THEN
          htp.print('api="'||o_api||'" ');
        END IF;
--        
        IF (o_name != '-99') THEN
          SELECT REPLACE(o_name,', ',' ')
            INTO o_name
            FROM DUAL;
          htp.print('name="'||o_name||'" ');
        END IF;
--        
        IF (o_status != '-99') THEN
          htp.print('status="'||o_status||'" ');
        END IF;
--
        htp.print('/>');
--
--    Other well information
--
--      IF ((o_field != '-99')   OR (o_field_kid != -99) OR
--          (o_operator!= '-99') OR (o_oper_kid != -99)) THEN
--            
--        htp.print('    <other ');
--          
--        IF (o_operator != '-99') THEN
--          htp.print('operator="'||o_operator||'" ');
--        END IF;       
--          
--        IF (o_oper_kid != -99) THEN
--          htp.print('oper_kid="'||o_oper_kid||'" ');
--        END IF;
--          
--        IF (o_field != '-99') THEN
--          htp.print('field="'||o_field||'" ');
--        END IF;
--          
--        IF (o_field_kid != -99) THEN
--          htp.print('field_kid="'||o_field_kid||'" ');
--        END IF;
--
--        htp.print('/>');
--      END IF;
--
--    Location Information
--
--      IF ((o_state_cd != -9)    OR (o_state != '-99')   OR 
--          (o_county_cd != -99)  OR (o_county != '-99')  OR 
--          (o_location != '-99') OR 
      IF  ((o_township != -99)   OR (o_town_dir != 'X')  OR 
          (o_range != -99)      OR (o_range_dir != 'X') OR 
          (o_section != -9)) THEN
--           
        htp.print('    <loc ');
--          
--        IF (o_state != '-99') THEN
--          htp.print('state="'||o_state||'" ');
--        END IF;       
--          
--        IF (o_state_cd != -9) THEN
--          htp.print('state_cd="'||o_state_cd||'" ');
--        END IF;       
--          
--        IF (o_county != '-99') THEN
--          htp.print('county="'||o_county||'" ');
--        END IF;       
--          
--        IF (o_county_cd != -99) THEN
--          htp.print('county_cd="'||o_county_cd||'" ');
--        END IF;       
--          
--        IF (o_location != '-99') THEN
--          htp.print('loc="'||o_location||'" ');
--        END IF;       
--          
        IF (o_township != -99) THEN
          htp.print('town="'||o_township||'" ');
        END IF;       
--          
        IF (o_town_dir != 'X') THEN
          htp.print('town_dir="'||o_town_dir||'" ');
        END IF;       
--          
        IF (o_range != -99) THEN
          htp.print('range="'||o_range||'" ');
        END IF;       
--          
        IF (o_range_dir != 'X') THEN
          htp.print('range_dir="'||o_range_dir||'" ');
        END IF;       
--          
        IF (o_section != -9) THEN
          htp.print('section="'||o_section||'" ');
        END IF;       
--
        htp.print('/>');
      END IF;
--
--    XY Position Information
--
      IF ((o_latitude != -99) OR (o_longitude != -99) OR 
          (o_utm_x != -99) OR (o_utm_y != -99)  OR (o_utm_zone != -99)) THEN
--            
        htp.print('    <xy ');
--          
        IF (o_latitude != -99) THEN
          htp.print('latitude="'||o_latitude||'" ');
        END IF;       
--          
        IF (o_longitude != -99) THEN
          htp.print('longitude="'||o_longitude||'" ');
        END IF;       
--          
        IF (o_utm_zone != -99) THEN
          htp.print('zone="'||o_utm_zone||'" ');
        END IF;       
--          
        IF (o_utm_x != -99) THEN
          htp.print('utm_x="'||o_utm_x||'" ');
        END IF;       
--          
        IF (o_utm_y != -99) THEN
          htp.print('utm_y="'||o_utm_y||'" ');
        END IF;       
--
        htp.print('/>');
      END IF;
--
--    Z Position - Depth, Elevation
--
      IF ((o_depth != -99) OR (o_kb != -99) OR (o_gl != -99) OR (o_df != -99)) THEN
--            
        htp.print('    <z ');
--          
        IF (o_depth != -99) THEN
          htp.print('depth="'||o_depth||'" ');
        END IF;       
--          
        IF (o_gl != -99) THEN
          htp.print('gl="'||o_gl||'" ');
        END IF;       
--          
        IF (o_kb != -99) THEN
          htp.print('kb="'||o_kb||'" ');
        END IF;       
--          
        IF (o_df != -99) THEN
          htp.print('df="'||o_df||'" ');
        END IF;       
--
        htp.print('/>');
      END IF;
--      
--    Comments
--
--      IF (o_comments != '-99') THEN
--        htp.print('    <comments> '||o_comments||'</comments>');
--      END IF;
--
--    Miscellaneous Fields
--
--      htp.print('    <misc ');
--          
--      IF (o_by != '-99') THEN
--        htp.print('user="'||o_by||'" ');
--      END IF;       
--         
--      IF (o_date != '-99') THEN
--        htp.print('date="'||o_date||'" ');
--      END IF;       
--
--      htp.print('/>');
--
      l_cnt := 0;
      SELECT COUNT(*)
        INTO l_cnt
        FROM las.well_headers
       WHERE well_header_kid IN (SELECT kid
                                   FROM co2_brine_well
                                  WHERE kid = o_kid);
--
      t_cnt := 0;
      SELECT COUNT(*)
        INTO t_cnt
        FROM qualified.well_tops
       WHERE well_header_kid IN (SELECT kid
                                   FROM co2_brine_well
                                  WHERE kid = o_kid);
--
      c_cnt := 0;
      SELECT COUNT(*)
        INTO c_cnt
        FROM core_library.core_data_and_descriptions
       WHERE well_header_kid IN (SELECT kid
                                   FROM co2_brine_well
                                  WHERE kid = o_kid)
         AND depth_top_core_ft IS NOT NULL;
--
      i_cnt := 0;
      SELECT COUNT(*)
        INTO i_cnt
        FROM qualified.images
       WHERE well_header_kid IN (SELECT kid
                                   FROM co2_brine_well
                                  WHERE kid = o_kid)
         AND file_url IS NOT NULL;
--
      htp.print('    <cnt  las="'||l_cnt||'" '||
                          'tops="'||t_cnt||'" '||
                          'core="'||c_cnt||'" '||
                          'images="'||i_cnt||'"/>');
--        
      htp.print('  </data>');
    END LOOP;
    CLOSE crsr;
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.print(' <data>');
      htp.print('    <info error="'||SQLERRM||'"/>');
      htp.print(' </data>');
--
  END getDataKGS;
--
--
-- PROCEDURE    getData()
-- Description: This procedure will retrieve headers information from the
--              headers table and construct a XML.
-- Input:       sKID = Headers Primary Key of the header information data 
--              sKEY    = User Created Primary Key
--              sUser   = User Primary Key
--              iAccess = User Access to data
--
  PROCEDURE getData(sKID IN VARCHAR2) IS
--
    l_count      NUMBER;
--    
    o_kid        co2_brine_well.kid%TYPE;        -- Primary Key for table
    o_name       VARCHAR(200);                           -- Well Name or Outcrop Name
    o_status     iqstrat.co2_brine_wells.status%TYPE;     -- well status or Outcrop Type  
-- Location Columns
    o_state_cd   iqstrat.co2_brine_wells.state_code%TYPE;   -- State Code
    o_state      global.states.name%TYPE;                  -- State Name
    o_county_cd  iqstrat.co2_brine_wells.county_code%TYPE;  -- County Code
    o_county     global.states_counties.county_name%TYPE;  -- County Name
    o_location   headers.location%TYPE;                    -- Location 
    o_township   iqstrat.co2_brine_wells.township%TYPE;           -- Township
    o_town_dir   iqstrat.co2_brine_wells.township_direction%TYPE; -- Township direction
    o_range      iqstrat.co2_brine_wells.range%TYPE;              -- Range
    o_range_dir  iqstrat.co2_brine_wells.range_direction%TYPE;    -- Range direction
    o_section    iqstrat.co2_brine_wells.section%TYPE;            -- Section
-- XY Position of Columns
    o_latitude   iqstrat.co2_brine_wells.nad27_latitude%TYPE;     -- Latitude in decimal degrees
    o_longitude  iqstrat.co2_brine_wells.nad27_longitude%TYPE;    -- Longitude in decimal degrees
    o_utm_x      iqstrat.co2_brine_wells.nad27_utm_x%TYPE;        -- UTM X-Coordinate
    o_utm_y      iqstrat.co2_brine_wells.nad27_utm_y%TYPE;        -- UTM Y-Coordinate
    o_utm_zone   iqstrat.co2_brine_wells.nad27_utm_zone%TYPE;     -- UTM Zone
-- Z Position - Depth, Elevation Columns
    o_depth      iqstrat.co2_brine_wells.rotary_total_depth%TYPE;  -- depth of well or height of outcrop
    o_kb         iqstrat.co2_brine_wells.elevation_kb%TYPE;    -- elevation kelly bushing (well)
    o_gl         iqstrat.co2_brine_wells.elevation_gl%TYPE;    -- elevation ground level
    o_df         iqstrat.co2_brine_wells.elevation_df%TYPE;    -- elevation derrick floor (well)
--
    CURSOR crsr (sKID VARCHAR2) IS
      SELECT kid,                    
             NVL(REPLACE(name,'&','&amp;'), '-99'),
             NVL(REPLACE(status,'&','&amp;'), '-99'),  
             NVL(state_code, -9),
             NVL((SELECT name 
                    FROM global.states 
                   WHERE code = A.state_code), '-99'), 
             NVL(county_code, -99),  
             NVL((SELECT county_name 
                    FROM global.states_counties 
                   WHERE state_code = A.state_code
                     AND county_code = A.county_code), '-99'), 
             NVL(township, -99),  
             NVL(township_direction, 'X'), 
             NVL(range, -99),  
             NVL(range_direction, 'X'), 
             NVL(section, -9),
             NVL(nad27_latitude, -99), 
             NVL(nad27_longitude, -99), 
             NVL(nad27_utm_x, -99), 
             NVL(nad27_utm_y, -99), 
             NVL(nad27_utm_zone, -99), 
             NVL(rotary_total_depth, -99),    
             NVL(elevation_kb, -99),
             NVL(elevation_gl, -99), 
             NVL(elevation_df, -99)
        FROM co2_brine_wells A
       WHERE kid = sKID;
--
  BEGIN
--
    o_kid       := '-99';
    o_name      := '-99';  -- Well Name or Outcrop Name
    o_status    := '-99';  -- well status or Outcrop Type  
-- Location Columns
    o_state_cd  := -9;     -- State Code
    o_state     := '-99';  -- State Name
    o_county_cd := -99;    -- County Code
    o_county    := '-99';  -- County Name
    o_location  := '-99';  -- Location 
    o_township  := -99;    -- Township
    o_town_dir  := 'X';    -- Township direction
    o_range     := -99;    -- Range
    o_range_dir := 'X';    -- Range direction
    o_section   := -9;     -- Section
-- XY Position of Columns
    o_latitude  := -99;    -- Latitude in decimal degrees
    o_longitude := -99;    -- Longitude in decimal degrees
    o_utm_x     := -99;    -- UTM X-Coordinate
    o_utm_y     := -99;    -- UTM Y-Coordinate
    o_utm_zone  := -99;    -- UTM Zone
-- Z Position - Depth, Elevation Columns
    o_depth     := -99;    -- depth of well or height of outcrop
    o_kb        := -99;    -- elevation kelly bushing (well)
    o_gl        := -99;    -- elevation ground level
    o_df        := -99;    -- elevation derrick floor (well)
--  
    OPEN crsr(sKID);
    LOOP
      FETCH crsr
       INTO o_kid,       -- Primary Key for table
            o_name,      -- Well Name or Outcrop Name
            o_status,    -- well status or Outcrop Type  
            -- Location Columns
            o_state_cd,  -- State Code
            o_state,     -- State Name
            o_county_cd, -- County Code
            o_county,    -- County Name
            o_township,  -- Township
            o_town_dir,  -- Township direction
            o_range,     -- Range
            o_range_dir, -- Range direction
            o_section,   -- Section
            -- XY Position of Columns
            o_latitude,  -- Latitude in decimal degrees
            o_longitude, -- Longitude in decimal degrees
            o_utm_x,     -- UTM X-Coordinate
            o_utm_y,     -- UTM Y-Coordinate
            o_utm_zone,  -- UTM Zone
            -- Z Position - Depth, Elevation Columns
            o_depth,     -- depth of well or height of outcrop
            o_kb,        -- elevation kelly bushing (well)
            o_gl,        -- elevation ground level
            o_df;        -- elevation derrick floor (well)
      EXIT WHEN crsr%NOTFOUND;
--
--      Identification Information
--
        htp.print('  <data>');
        htp.print('    <info kid="'||o_kid||'" '||
                            'well_kid="'||o_kid||'" ');
--              
        IF (o_name != '-99') THEN
          htp.print('name="'||o_name||'" ');
        END IF;
--        
        IF (o_status != '-99') THEN
          htp.print('status="'||o_status||'" ');
        END IF;
--
        htp.print('/>');
--
--    Location Information
--
      IF ((o_state_cd != -9)    OR (o_state != '-99')   OR 
          (o_county_cd != -99)  OR (o_county != '-99')  OR 
          (o_location != '-99') OR 
          (o_township != -99)   OR (o_town_dir != 'X')  OR 
          (o_range != -99)      OR (o_range_dir != 'X') OR 
          (o_section != -9)) THEN
--            
        htp.print('    <loc ');
--          
        IF (o_state != '-99') THEN
          htp.print('state="'||o_state||'" ');
        END IF;       
--          
        IF (o_state_cd != -9) THEN
          htp.print('state_cd="'||o_state_cd||'" ');
        END IF;       
--          
        IF (o_county != '-99') THEN
          htp.print('county="'||o_county||'" ');
        END IF;       
--          
        IF (o_county_cd != -99) THEN
          htp.print('county_cd="'||o_county_cd||'" ');
        END IF;       
--          
        IF (o_location != '-99') THEN
          htp.print('loc="'||o_location||'" ');
        END IF;       
--          
        IF (o_township != -99) THEN
          htp.print('town="'||o_township||'" ');
        END IF;       
--          
        IF (o_town_dir != 'X') THEN
          htp.print('town_dir="'||o_town_dir||'" ');
        END IF;       
--          
        IF (o_range != -99) THEN
          htp.print('range="'||o_range||'" ');
        END IF;       
--          
        IF (o_range_dir != 'X') THEN
          htp.print('range_dir="'||o_range_dir||'" ');
        END IF;       
--          
        IF (o_section != -9) THEN
          htp.print('section="'||o_section||'" ');
        END IF;       
--
        htp.print('/>');
      END IF;
--
--    XY Position Information
--
      IF ((o_latitude != -99) OR (o_longitude != -99) OR 
          (o_utm_x != -99) OR (o_utm_y != -99)  OR (o_utm_zone != -99)) THEN
--            
        htp.print('    <xy ');
--          
        IF (o_latitude != -99) THEN
          htp.print('latitude="'||o_latitude||'" ');
        END IF;       
--          
        IF (o_longitude != -99) THEN
          htp.print('longitude="'||o_longitude||'" ');
        END IF;       
--          
        IF (o_utm_zone != -99) THEN
          htp.print('zone="'||o_utm_zone||'" ');
        END IF;       
--          
        IF (o_utm_x != -99) THEN
          htp.print('utm_x="'||o_utm_x||'" ');
        END IF;       
--          
        IF (o_utm_y != -99) THEN
          htp.print('utm_y="'||o_utm_y||'" ');
        END IF;       
--
        htp.print('/>');
      END IF;
--
--    Z Position - Depth, Elevation
--
      IF ((o_depth != -99) OR (o_kb != -99) OR (o_gl != -99) OR (o_df != -99)) THEN
--            
        htp.print('    <z ');
--          
        IF (o_depth != -99) THEN
          htp.print('depth="'||o_depth||'" ');
        END IF;       
--          
        IF (o_gl != -99) THEN
          htp.print('gl="'||o_gl||'" ');
        END IF;       
--          
        IF (o_kb != -99) THEN
          htp.print('kb="'||o_kb||'" ');
        END IF;       
--          
        IF (o_df != -99) THEN
          htp.print('df="'||o_df||'" ');
        END IF;       
--
        htp.print('/>');
      END IF;
--      
      htp.print('  </data>');
    END LOOP;
    CLOSE crsr;
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.print(' <data>');
      htp.print('    <info error="'||SQLERRM||'"/>');
      htp.print(' </data>');
--
  END getData;
--
--
END co2_gridding_pkg;
/
SHOW ERRORS;
--
GRANT EXECUTE ON  co2_gridding_pkg to apex_public_user;
