-- * PLSQL:       co2_production_pkg.sql Version 1.1 3/30/2016
-- * Author:      John R. Victorine
-- * Description: This Package will retrieve the Production data for a
-- *              a specific lease, field, operator or county.
-- * Copyright (c) 2016 Kansas Geological Survey
-- * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
-- * All Rights Reserved.
--
SET SCAN OFF;
--
CREATE OR REPLACE PACKAGE co2_production_pkg AS
  PROCEDURE getXML(sFluid IN VARCHAR2);
  PROCEDURE add_data_record_to_body(sYear IN VARCHAR2,
                                    sMonth IN VARCHAR2,
                                    sDay   IN VARCHAR2,
                                    sDate  IN VARCHAR2,
                                    sProduct IN VARCHAR2,
                                    sProduction IN VARCHAR2,
                                    sUnits IN VARCHAR2,
                                    sWells IN VARCHAR2);
  PROCEDURE build_DTD;
END co2_production_pkg;
/
--
CREATE OR REPLACE PACKAGE BODY co2_production_pkg AS
--
-- PROCEDURE    getXML
-- Description: This procedure will build the Production data for the selected
--              Lease KID.
-- Input:       sFluid = Use Well KID from CO2 Fluid Database
--
  PROCEDURE getXML(sFluid IN VARCHAR2) IS
--
    l_count      NUMBER(6);     -- Total Number of Production Records
    l_count1     NUMBER(6);     -- Total Number of Production Records
    l_count2     NUMBER(6);     -- Total Number of Production Records
    o_api        co2_brine_well.api_number%TYPE;
    o_lease      brine.samples.lease_name%TYPE;
--
    o_date       VARCHAR2(40);
    o_year       VARCHAR2(5);
    o_month      VARCHAR2(3);
    o_day        VARCHAR2(3);
    o_product    VARCHAR2(4);
    o_gas        iqstrat.co2_and_offset_gas.gas_mcf_per_day%TYPE;
    o_oil        iqstrat.co2_producer_test_data.barrels_of_oil_per_day%TYPE;
    o_units      VARCHAR(20);
    o_wells      NUMBER(2);
--
  CURSOR gas_crsr (sKID IN VARCHAR2) IS
      SELECT TO_CHAR(measured_date, 'YYYY') AS YEAR,         -- Year the Record is recorded
             TO_CHAR(measured_date, 'MM') AS MONTH,          -- Month the Record is recorded
             NVL(TO_CHAR(measured_date, 'DD'),'-99') AS DAY, -- Day the Record is recorded
             TO_CHAR(measured_date, 'MM/DD/YYYY HH24:MI:SS'),
             'GAS',                                          -- Product being reported, GAS
             gas_mcf_per_day,                                -- The Actual Production (MCF)
             'MCF',
             1
        FROM co2_and_offset_gas
       WHERE well_kid = sKID
         AND gas_mcf_per_day > 0
      ORDER BY YEAR, MONTH, DAY;
--
  CURSOR oil_crsr (sKID IN VARCHAR2) IS
      SELECT TO_CHAR(test_date, 'YYYY') AS YEAR,         -- Year the Record is recorded
             TO_CHAR(test_date, 'MM') AS MONTH,          -- Month the Record is recorded
             NVL(TO_CHAR(test_date, 'DD'),'-99') AS DAY, -- Day the Record is recorded
             TO_CHAR(test_date, 'MM/DD/YYYY HH24:MI:SS'),
             'OIL',                                      -- Product being reported, GAS
             barrels_of_oil_per_day,                     -- The Actual Production (BBL)
             'BBL',
             1
        FROM co2_producer_test_data
       WHERE well_kid = sKID
         AND barrels_of_oil_per_day > 0
      ORDER BY YEAR, MONTH, DAY;
--
  BEGIN
--
    SELECT COUNT(kid)
      INTO l_count
      FROM co2_brine_well
     WHERE kid = sFluid;  
--
    IF (l_count > 0) THEN
      SELECT REPLACE(lease_name||' '||well_name,', ',' '),
             NVL(api_number, '-999')
        INTO o_lease,
             o_api
        FROM co2_brine_well
       WHERE kid = sFluid;  
    ELSE
      o_api := '0';
      SELECT name
        INTO o_lease
        FROM co2_brine_wells
       WHERE kid = sFluid;  
    END IF;
--       
    SELECT COUNT(gas_mcf_per_day)
      INTO l_count1
      FROM co2_and_offset_gas
     WHERE well_kid = sFluid
       AND gas_mcf_per_day > 0;
--
    l_count := l_count1;
--
    SELECT COUNT(barrels_of_oil_per_day)
      INTO l_count2
      FROM co2_producer_test_data
     WHERE well_kid = sFluid
       AND barrels_of_oil_per_day > 0;
--
    l_count := l_count + l_count2;
--
--  Add the DTD to the Top of the XML File
--
    build_DTD;
--
--  Open the XML Body of Production data
--
    htp.print('<production records="'||l_count||'" ');
    htp.print('            key="'||sFluid||'" ');
    htp.print('            type="FLUID" ');
    htp.print('            name="'||INITCAP(o_lease)||'" ');
    IF (o_api != '-999') THEN
      htp.print('            api="'||o_api||'" ');
    END IF;
    htp.print('            app="http://chasm.kgs.ku.edu/ords/'||
                               'iqstrat.co2_co2_production_pkg.getXML" ');
    htp.print('            v1="sfluid='||sFluid||'">');
--
    OPEN gas_crsr(sFluid);
    LOOP
      FETCH gas_crsr
       INTO o_year,
            o_month,
            o_day,
            o_date,
            o_product,
            o_gas,
            o_units,
            o_wells;
      EXIT WHEN gas_crsr%NOTFOUND;
--
--    ADD EACH Production Record to XML
--
      add_data_record_to_body(o_year, o_month, o_day, o_date, o_product, o_gas, o_units, o_wells);
--
    END LOOP;
    CLOSE gas_crsr;
--
    OPEN oil_crsr(sFluid);
    LOOP
      FETCH oil_crsr
       INTO o_year,
            o_month,
            o_day,
            o_date,
            o_product,
            o_oil,
            o_units,
            o_wells;
      EXIT WHEN oil_crsr%NOTFOUND;
--
--    ADD EACH Production Record to XML
--
      add_data_record_to_body(o_year, o_month, o_day, o_date, o_product, o_oil, o_units, o_wells);
--
    END LOOP;
    CLOSE oil_crsr;
--
--  Close the XML Body of the Production Data
--
    htp.print('</production>');
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.p('iqstrat.co2_production_pkg.getXML(LEASE KID)'||
            'An error occurred: '||SQLERRM||'. Please try again later.');
  END getXML;
--
--
-- PROCEDURE    add_data_record_to_body
-- Description: This function will add a record to the body of the XML
-- Input:      sYear       -- Year the Production Record is recorded
--             sMonth      -- Month the Production Record is recorded
--             sDay        -- DAY the Production Record is recorded
--             sProduct    -- Product being reported, Oil or Gas
--             sProduction -- The Actual Production BBL or MCF
--             sWells      -- Number of Wells contributing to production
--
  PROCEDURE add_data_record_to_body(sYear IN VARCHAR2,
                                    sMonth IN VARCHAR2,
                                    sDay   IN VARCHAR2,
                                    sDate  IN VARCHAR2,
                                    sProduct IN VARCHAR2,
                                    sProduction IN VARCHAR2,
                                    sUnits IN VARCHAR2,
                                    sWells IN VARCHAR2) IS
--
    o_temp       VARCHAR(4096);
--
  BEGIN
--
    o_temp := '  <data year="'||TO_NUMBER(sYear)||'" '||
                      'month="'||TO_NUMBER(sMonth)||'" ';
--                      
    IF (sDAY != '-99') THEN
      o_temp := o_temp||'day="'||TO_NUMBER(sDay)||'" ';
    END IF;
--    
    o_temp := o_temp||'date="'||sDate||'" '||
                      'product="'||sProduct||'" '||
                      'output="'||sProduction||'" '||
                      'unit="'||sUnits||'" '||
                      'sources="'||sWells||'"/>';
    htp.print(o_temp);
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.p('iqstrat.co2_production_pkg.buildDTD()'||
            'An error occurred: '||SQLERRM||'. Please try again later.');
  END add_data_record_to_body;
--
--
-- PROCEDURE    build_DTD
-- Description: This function will return the DTD for the Production
--
  PROCEDURE build_DTD IS
--
--
  BEGIN
--
    htp.print('<?xml version="1.0"?>');
    htp.print('<!DOCTYPE production [');
    htp.print('<!ELEMENT production (data*)>');
    htp.print('<!ATTLIST production records CDATA #REQUIRED'); -- // Total # of Records
    htp.print('                     key     CDATA #REQUIRED'); -- // Primary Key
    htp.print('                     type    CDATA #REQUIRED'); -- // Type of Data
    htp.print('                     name    CDATA #REQUIRED'); -- // Name of Data
    htp.print('                     api     CDATA #IMPLIED');  -- // API-Number
    htp.print('                     app     CDATA #IMPLIED');  -- // XML Application
    htp.print('                     v1      CDATA #IMPLIED');  -- // Variable 1
    htp.print('                     v2      CDATA #IMPLIED');  -- // Variable 2
    htp.print('                     v3      CDATA #IMPLIED>'); -- // Variable 3
    htp.print('<!ELEMENT data  EMPTY>');                       --
    htp.print('<!ATTLIST data  year    CDATA #REQUIRED');   -- // Year Number
    htp.print('                month   CDATA #IMPLIED');    -- // Month Number 0,1,2,...12
    htp.print('                day     CDATA #IMPLIED');    -- // Day Number 1,2,3,4
    htp.print('                quarter CDATA #IMPLIED');    -- // Quarter Number 1,2,3...
    htp.print('                date    CDATA #IMPLIED');    -- // Date of record
    htp.print('                product CDATA #REQUIRED');   -- // Product, OIL, GAS, etc.
    htp.print('                unit    CDATA #IMPLIED');    -- // Units of Data BBL or MCF
    htp.print('                output  CDATA #REQUIRED');   -- // Production
    htp.print('                sources CDATA #IMPLIED>]>'); -- // Number of Wells
--
  EXCEPTION
    WHEN OTHERS THEN
      htp.p('iqstrat.co2_production_pkg.buildDTD()'||
            'An error occurred: '||SQLERRM||'. Please try again later.');
  END build_DTD;
--
--
END co2_production_pkg;
/
SHOW ERRORS;
--
GRANT EXECUTE ON  co2_production_pkg to apex_public_user;